@@ -76,22 +76,22 @@
 #                                                                             #
 ###############################################################################
 
+=encoding utf8
+
 =head1 NAME
 
-  CracTools::SAMReader::SAMline - The object for manipulation a SAM line.
+CracTools::SAMReader::SAMline - The object for manipulation a SAM line.
 
 =head1 SYNOPSIS
 
-Usage:
-
   use CracTools::SAMReader::SAMline;
 
   $sam_line = CracTools::SAMReader::SAMline->new($line);
 
 =head1 DESCRIPTION
 
-  An object for easy acces to SAM line fields. See SAM Specifications for more informations :
-  http://samtools.sourceforge.net/SAM1.pdf
+An object for easy acces to SAM line fields. See SAM Specifications for more informations :
+http://samtools.sourceforge.net/SAM1.pdf
 
 =cut
 
@@ -102,6 +102,7 @@ use strict;
 use warnings;
 use Carp;
 use Data::Dumper;
+use Switch;
 
 use CracTools::Utils;
 
@@ -129,12 +130,14 @@ SAM flags :
 
 =item * LAST_SEGMENT => 128,
 
-=item * SECONDARY_ALIGNEMENT => 256,
+=item * SECONDARY_ALIGNMENT => 256,
 
 =item * QUALITY_CONTROLS_FAILED => 512,
 
 =item * PCR_DUPLICATED => 1024,
 
+=item * CHIMERIC_ALIGNMENT => 2048,
+
 =back
 
 =cut
@@ -147,9 +150,10 @@ our %flags = ( MULTIPLE_SEGMENTS => 1,
             NEXT_REVERSE_COMPLEMENTED => 32,
             FIRST_SEGMENT => 64,
             LAST_SEGMENT => 128,
-            SECONDARY_ALIGNEMENT => 256,
+            SECONDARY_ALIGNMENT => 256,
             QUALITY_CONTROLS_FAILED => 512,
             PCR_DUPLICATED => 1024,
+            CHIMERIC_ALIGNMENT => 2048,
           );
 
 =head1 STATIC PARSING METHODS
@@ -189,21 +193,30 @@ sub hasEvent {
 sub new {
   my $class = shift;
   my ($line) = @_;
-  
-  my ($qname,$flag,$rname,$pos,$mapq,$cigar,$rnext,$pnext,$tlen,$seq,$qual) = split(/\s+/,$line);
+  chomp $line;  
+  my ($qname,$flag,$rname,$pos,$mapq,$cigar,$rnext,$pnext,$tlen,$seq,$qual,@others) = split("\t",$line);
+  my %extended_fields;
+  foreach(@others) {
+    my $key = substr($_,0,2);
+    my $value = substr($_,5);
+    #my($key,$type,$value) = split(':',$_,3); 
+    # Hack in case there is multiple field with the same tag
+    $extended_fields{$key} = defined $extended_fields{$key}? $extended_fields{$key}.";".$value : $value;
+  }
 
   my $self = bless{ 
-      qname => $qname,
-      flag => $flag,
-      rname => $rname,
-      pos => $pos,
-      mapq => $mapq,
-      cigar => $cigar,
-      rnext => $rnext,
-      pnext => $pnext,
-      tlen => $tlen,
-      seq => $seq,
-      qual => $qual,
+    qname => $qname,
+    flag => $flag,
+    rname => $rname,
+    pos => $pos,
+    mapq => $mapq,
+    cigar => $cigar,
+    rnext => $rnext,
+    pnext => $pnext,
+    tlen => $tlen,
+    seq => $seq,
+    qual => $qual,
+    extended_fields => \%extended_fields,
     line => $line,
   }, $class;
 
@@ -356,8 +369,6 @@ sub qname {
   my $new_qname = shift;
   if(defined $new_qname) {
     $self->{qname} = $new_qname;
-  } elsif(!defined $self->{qname}) {
-    $self->{qname} = $self->getField(0);
   }
   return $self->{qname};
 }
@@ -375,8 +386,6 @@ sub flag {
   my $new_flag = shift;
   if(defined $new_flag) {
     $self->{flag} = $new_flag;
-  } elsif(!defined $self->{flag}) {
-    $self->{flag} = $self->getField(1);
   }
   return $self->{flag};
 }
@@ -394,8 +403,6 @@ sub rname {
   my $new_rname = shift;
   if(defined $new_rname) {
     $self->{rname} = $new_rname;
-  } elsif(!defined $self->{rname}) {
-    $self->{rname} = $self->getField(2);
   }
   return $self->{rname};
 }
@@ -426,8 +433,6 @@ sub pos {
   my $new_pos = shift;
   if(defined $new_pos) {
     $self->{pos} = $new_pos;
-  } elsif(!defined $self->{pos}) {
-    $self->{pos} = $self->getField(3);
   }
   return $self->{pos};
 }
@@ -445,8 +450,6 @@ sub mapq {
   my $new_mapq = shift;
   if(defined $new_mapq) {
     $self->{mapq} = $new_mapq;
-  } elsif(!defined $self->{mapq}) {
-    $self->{mapq} = $self->getField(4);
   }
   return $self->{mapq};
 }
@@ -464,8 +467,6 @@ sub cigar {
   my $new_cigar = shift;
   if(defined $new_cigar) {
     $self->{cigar} = $new_cigar;
-  } elsif(!defined $self->{cigar}) {
-    $self->{cigar} = $self->getField(5);
   }
   return $self->{cigar};
 }
@@ -483,8 +484,6 @@ sub rnext {
   my $new_rnext = shift;
   if(defined $new_rnext) {
     $self->{rnext} = $new_rnext;
-  } elsif(!defined $self->{rnext}) {
-    $self->{rnext} = $self->getField(6);
   }
   return $self->{rnext};
 }
@@ -502,8 +501,6 @@ sub pnext {
   my $new_pnext = shift;
   if(defined $new_pnext) {
     $self->{pnext} = $new_pnext;
-  } elsif(!defined $self->{pnext}) {
-    $self->{pnext} = $self->getField(7);
   }
   return $self->{pnext};
 }
@@ -521,15 +518,15 @@ sub tlen {
   my $new_tlen = shift;
   if(defined $new_tlen) {
     $self->{tlen} = $new_tlen;
-  } elsif(!defined $self->{tlen}) {
-    $self->{tlen} = $self->getField(8);
   }
   return $self->{tlen};
 }
 
 =head2 seq
 
-  Description : Getter/Setter for attribute seq (the sequence)
+  Description : Getter/Setter for attribute seq (the sequence).
+                Please use getOriginalSeq if you want to retrieve the oriented
+                sequence, that what you need in most cases.
   ReturnType  : String
   Exceptions  : none
 
@@ -540,8 +537,6 @@ sub seq {
   my $new_seq = shift;
   if(defined $new_seq) {
     $self->{seq} = $new_seq;
-  } elsif(!defined $self->{seq}) {
-    $self->{seq} = $self->getField(9);
   }
   return $self->{seq};
 }
@@ -559,12 +554,167 @@ sub qual {
   my $new_qual = shift;
   if(defined $new_qual) {
     $self->{qual} = $new_qual;
-  } elsif(!defined $self->{qual}) {
-    $self->{qual} = $self->getField(10);
   }
   return $self->{qual};
 }
 
+=head2 getOptionalField
+
+  Example     : 
+  Description : 
+  ReturnType  : 
+
+=cut
+
+sub getOptionalField {
+  my $self = shift;
+  my $field = shift;
+  croak("Missing \$field argument to call getOptionalField") unless defined $field;
+  return $self->{extended_fields}{$field};
+}
+
+
+=head2 getChimericAlignments
+
+  Description : Parser of SA fields of SAM file in order to find chimeric reads
+  ReturnType  : Array reference
+                Elements are hash [ chr    => String, 
+                                    pos    => int, 
+                                    strand => 1/-1, 
+                                    cigar  => String,
+                                    mapq   => int,
+                                    edist  => int
+                                  ]
+
+=cut
+
+sub getChimericAlignments {
+    my $self = shift;
+    # check the existence of the SA field in the SAM line
+    if (defined $self->{extended_fields}{SA}){
+	my @array_hash;
+	my (@SA_alignments) = split(/;/,$self->{extended_fields}{SA});
+	for (my $i=0 ;  $i < scalar @SA_alignments ; $i++){
+	    my ($chr,$pos,$strand,$cigar,$mapq,$edist) = split(/,/,$SA_alignments[$i]);
+	    # strand switch from "+,-" to "1,-1"
+	    if ($strand eq '+'){
+		$strand = 1;
+	    }else{
+		$strand = -1;
+	    }
+	    my $hash = { chr => $chr, 
+			 pos => $pos,
+			 strand => $strand,
+			 cigar => $cigar,
+			 mapq => $mapq,
+			 edist => $edist};
+	    push(@array_hash,$hash);
+	}
+	return \@array_hash;
+    }
+    return undef;
+}
+
+=head2 getCigarOperatorsCount
+
+  Example     : my %cigar_counts = %{ $sam_line->getCigarOperatorsCount() };
+                print "nb mismatches; ",$cigar_counts{X},"\n";
+  Description : Return a hash reference where the keys are the cigar operators and the values
+                the sum of length associated for each operator.
+                For cigar 5S3M1X2M10S, getCigarOperatorsCounts() will retrun :
+                { 'S' => 15,
+                  'M' => 5,
+                  'X' => 1,
+                };
+  ReturnType  : Hash reference 
+=cut
+
+sub getCigarOperatorsCount {
+  my $self = shift;
+  my @ops = $self->cigar =~ /(\d+\D)/g;
+  my %ops_occ;
+  foreach (@ops) {
+    my ($nb,$op) = $_ =~ /(\d+)(\D)/;
+    $ops_occ{$op} = 0 unless defined $ops_occ{$op};
+    $ops_occ{$op} += $nb;
+  }
+  return \%ops_occ;
+}
+
+=head2 pSupport
+
+  Description : Return the support profile of the read if the SAM file has been generated with
+                CRAC option --detailed
+  ReturnType  : String
+
+=cut
+
+sub pSupport {
+  my $self = shift;
+  $self->loadSamDetailed;
+  return $self->{sam_detailed}{p_support};
+}
+
+=head2 pLoc
+
+  Description : Return the location profile of the read if the SAM file has been generated with
+                CRAC option --detailed
+  ReturnType  : String
+
+=cut
+
+sub pLoc {
+  my $self = shift;
+  $self->loadSamDetailed;
+  return $self->{sam_detailed}{p_loc};
+}
+
+=head2 pairedChimera
+
+  Description : return the chimeric coordinates of the paired chimera associated to this read if there is one
+
+  ReturnType  : array(chr1,pos1,strand1,chr2,pos2,strand2) or undef
+
+=cut
+
+sub pairedChimera {
+  my $self = shift;
+  $self->loadPaired();
+  if(defined $self->{extended_fields}{XP}{chimera}) {
+    my ($crac_loc1,$crac_loc2) = split(":",$self->{extended_fields}{XP}{chimera});
+    return (expandCracLoc($crac_loc1),expandCracLoc($crac_loc2));
+  } else {
+    return undef;
+  }
+}
+
+=head2 isPairedClassified
+
+  Arg [1] : String - The class to test :
+            - "unique"
+            - "duplicated"
+            - "multiple"
+
+  Description : Test paired-end read clasification
+
+  ReturnType  : Boolean
+
+=cut
+
+sub isPairedClassified {
+  my $self = shift;
+  my $class = shift;
+  $self->loadPaired();
+
+  if(defined $self->{extended_fields}{XP}{loc} && ref($self->{extended_fields}{XP}{loc}) ne 'HASH') {
+    my ($uniq,$dupli,$multi) = split(":",$self->{extended_fields}{XP}{loc});
+    $self->{extended_fields}{XP}{loc} = {unique => $uniq, duplicated => $dupli, multiple => $multi};
+
+  }
+  return $self->{extended_fields}{XP}{loc}{$class};
+}
+
+
 =head2 genericInfo
   
   [1] : Key of the generic info
@@ -614,11 +764,14 @@ sub genericInfo {
 sub isClassified {
   my $self = shift;
   my $class = shift;
-  $self->loadClassification();
-  if(defined $self->{classification}{$class} && $self->{classification}{$class} == 1) {
-    return 1;
-  } else {
-    return 0;
+
+  switch($class) {
+    case "unique"       { return $self->{extended_fields}{XU} }
+    case "duplicated"   { return $self->{extended_fields}{XD} }
+    case "multiple"     { return $self->{extended_fields}{XM} }
+    case "normal"       { return $self->{extended_fields}{XN} == 1 }
+    case "almostNormal" { return $self->{extended_fields}{XU} == 2 }
+    else                { return undef }
   }
 }
 
@@ -647,7 +800,7 @@ sub isClassified {
 sub events {
   my $self = shift;
   my $event_type = shift;
-  $self->loadEvents();
+  $self->loadEvents($event_type);
   if(defined $self->{events}{$event_type}) {
     return $self->{events}{$event_type};
   } else {
@@ -657,29 +810,6 @@ sub events {
 
 =head1 PRIVATE METHODS
 
-=head2 loadClassification
-
-  Example     : $sam_line->loadClassification();
-  Description : Loading of classification attributes
-  ReturnType  : none
-  Exceptions  : none
-
-=cut
-
-sub loadClassification {
-  my $self = shift;
-  if(!defined $self->{classification}) {
-    # Init classification
-    my %classification;
-    $classification{unique} = $self->line =~ /XU:i:1/;
-    $classification{duplicated} = $self->line =~ /XD:i:1/;
-    $classification{multiple} = $self->line =~ /XM:i:1/;
-    $classification{normal} = $self->line =~ /XN:i:1/;
-    $classification{almostNormal} = $self->line =~ /XN:i:2/;
-    $self->{classification} = \%classification;
-  }
-}
-
 =head2 loadEvents
 
   Example     : $sam_line->loadEvents();
@@ -692,13 +822,13 @@ sub loadClassification {
 sub loadEvents {
   my $self = shift;
   my $event_type_to_load = shift;
-  # TODO avoid double loading when doing lazy loading
+  ## TODO avoid double loading when doing lazy loading
   if(defined $event_type_to_load && defined $self->{$event_type_to_load}{loaded}) {
     return 0;
   }
-  if(!defined $self->{events}) { 
+  if(!defined $self->{events} && defined $self->{extended_fields}{XE}) { 
     # Init events
-    my @events = $self->line =~ /XE:Z:([^\t]+)/g;
+    my @events = split(";",$self->{extended_fields}{XE});
     foreach my $event (@events) {
       my ($event_id,$event_break_id,$event_type,$event_infos) = $event =~ /([^:]+):([^:]+):([^:]+):(.*)/g;
       next if(defined $event_type_to_load && $event_type ne $event_type_to_load);
@@ -788,7 +918,7 @@ sub addEvent {
   my $event = shift;
   my $event_type = $event->{event_type};
   if(defined $self->{events}{$event_type}) {
-    push($self->{events}{$event_type},$event);
+    push(@{$self->{events}{$event_type}},$event);
   } else {
     $self->{events}{$event_type} = [$event];
   }
@@ -797,6 +927,7 @@ sub addEvent {
 =head2 removeEvent 
 
   Arg [1] : Hash reference - The event object
+
   Description : Remove the event from the event hash and from the line.
 
 =cut
@@ -908,7 +1039,8 @@ sub updateEvent {
 sub loadSamDetailed {
   my $self = shift;
   if(!defined $self->{sam_detailed}) {
-    my ($detailed) = $self->line =~ /XR:Z:([^\s]+)/g;
+    #my ($detailed) = $self->line =~ /XR:Z:([^\s]+)/g;
+    my $detailed = $self->{extended_fields}{XR};
     my @detailed_fields = split(";",$detailed); 
     foreach (@detailed_fields) {
       my ($k,$v) = split('=',$_);
@@ -924,32 +1056,38 @@ sub loadSamDetailed {
   }
 }
 
-sub pSupport {
+=head2 loadPaired
+
+  Example     : $sam_line->loadPaired();
+  Description : Loading of sam detaileds attributes
+  ReturnType  : none
+  Exceptions  : none
+
+=cut
+
+sub loadPaired {
   my $self = shift;
-  $self->loadSamDetailed;
-  return $self->{sam_detailed}{p_support};
+  # If XP fields exist and we haven't load it already
+  if(defined $self->{extended_fields}{XP} && ref($self->{extended_fields}{XP}) ne 'HASH') {
+    my @paired_fields = split(";",$self->{extended_fields}{XP});
+    $self->{extended_fields}{XP} = {}; # Chamge type of XP exetended field from scalar to hash ref
+    foreach (@paired_fields) {
+      my($key,$value) = split(":",$_,2);
+      $self->{extended_fields}{XP}{$key} = $value;
+    }
+  }
 }
 
-sub pLoc {
-  my $self = shift;
-  $self->loadSamDetailed;
-  return $self->{sam_detailed}{p_loc};
-}
+=head2 expandCracLoc
 
-sub getField {
-  my $self = shift;
-  my $col_num = shift;
-  my @fields = split("\t",$self->line,$col_num+2);
-  return $fields[$col_num];
-  #my $sep = '\t';
-  #my $regex = '';
-  #for(my $i = 0; $i < $col_num; $i++) {
-  #  $regex .= '[^'.$sep.'.]*'.$sep;
-  #}
-  #$regex .= '([^'.$sep.'.]*)';
-  #my ($col_val) = $self->{line} =~ /^$regex/;
-  #return $col_val;
-}
+  Arg [1] : String - Localisation in crac format : Chromosome|strand,position
+            Ex : X|-1,2332377
+
+  Description : Extract Chromosme, position and strand as separated variable from
+                the localisation in CRAC format.
+  ReturnType  : Array($chromosome,$position,$strand)
+
+=cut
 
 sub expandCracLoc {
   my $loc = shift;
@@ -957,6 +1095,17 @@ sub expandCracLoc {
   return ($chr,$pos,$strand);
 }
 
+=head2 compressCracLoc
+
+  Arg [1] : String - Chromosome
+  Arg [2] : Integer - Postition
+  Arg [3] : Integer (1,-1) - Strand
+
+  Description : Reverse function of "expandCracLoc"
+  ReturnType  : String (localisation in CRAC format)
+
+=cut
+
 sub compressCracLoc {
   my ($chr,$pos,$strand) = @_;
   confess("Missing argument") unless defined $chr && defined $pos && defined $strand;
